// --- [ vnc_spi.c ] ----------------------------------------------------------
//
// tab = 3
// ----------------------------------------------------------------------------

#include <avr/io.h>
#include <inttypes.h>
#include <stdlib.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "global.h"
#include "rs232.h"
#include "vnc_spi.h"
#include "spi_13bit.h"


// --- [ lokale Variablen und Funktionen ] ------------------------------------
														// default-Dateiname, nur die 3 Ziffern
														// werden variiert. Lnge immer 13 Zeichen !
char VNC_filename[13] = "file_000.txt";	// 8 + 1 + 3 = 12 + '0' = 13
char VNC_dirname[9]   = "DIR__000";			// default-Verzeichnisnamen

// interne Funktionen
uint8_t VNC_switch_SCS(void);				 	// Microcontroller-Befehlssatz
uint8_t VNC_wait_for_prompt(void);			// auf das Prompt nach einem Befehl warten
uint8_t VNC_synch(void);						// ein Synch senden und Antwort auswerten
uint8_t VNC_send_cmd(uint8_t);				// Sendet ein 1 Byte Kommando via SCS
void VNC_spi_write_s(char *string);			// schreibt einen String (Dateinamen)

// ----------------------------------------------------------------------------
void VNC_spi_init(void)
{
	// Der Timer0 wird im Clear Timer on Compare Match Modus betrieben
	// Sobald der TCTN0 den Wert in OCR0A berschreiten, wird er TCNT0
	// auf 0 zurckgesetzt und eine ISR angesprungen.
	// Hier wird allgemein die Zeit aktualisiert sowie die Variable VNC_timeout decrementiert
	TCCR0A  = (1<<WGM01);						// CTC-Modus
	TCCR0B  = (1<<CS02 | 1<< CS00);			// prescale 1024
	TIMSK0  = (1<<OCIE0A);						// enable COMPA
	OCR0A   = 120;
	
	SPI_DDR |= (1<<SPI_CS | 1<< SPI_SDI | 1<<SPI_SCLK);
	RST_DDR |= (1<<SPI_RST);
	
	VNC_error = VNC_NO_ERR;						// Vorgabe kein Fehler
	
  	R_CS_VNC;  										// CS inaktiv
  	R_SCLK;    										// Ruhezustand der SPI-Leitungen herstellen
  	S_SDI;
  	R_RE_VNC;  										// Hardware-Reset VDIP

  	VNC_timeout = 5;								// ca. 1/6 Sekunde warten
	while (VNC_timeout);
	
  	S_RE_VNC;  										// VDIP starten
	
	VNC_timeout = 120;							// alles was VDIP in der nchsten 4 Sekunde zu melden hat
	while (VNC_spi_read() || VNC_timeout);	// auslesen und vergessen
	
VNC_firmware_version();

#ifdef DEBUG
	put_c(13);
	put_s("SCS: ");
#endif
	VNC_synch();									// Synchronisieren ggf. mit Wartezeit
	VNC_switch_SCS();								// Umschalten in Short Command Set
}

// ----------------------------------------------------------------------------
// Gibt die Firmware-Version via RS232 aus (nur zum Debuggen gedacht)
// Muss aufgerufen werden, bevor in den SCS-Modus geschaltet wird.
void VNC_firmware_version(void)
{
uint8_t i = 0;
VNC_spi_write('F');								// Sendet den String 'FWV'
VNC_spi_write('W');
VNC_spi_write('V');
VNC_spi_write(CHR_RETURN);

VNC_timeout = 120;								// 4 Sekunden Wartezeit
while (!(spi_in == CHR_PROMPT))				// alles bis zum Prompt einlesen und ablegen
	{
	if (VNC_spi_read()) vnc_buffer[i++] = spi_in;
	if (!(VNC_timeout)) break;
	if (i & 0x80) break;							// wenn mehr als 128 Zeichen empfangen wurden
	}													// dann liegt ein Fehler vor
vnc_buffer[i] = 0;								// den String terminieren
put_s(vnc_buffer);								// und absenden - die Warnung kann ignoriert werden ! 
put_c(CHR_RETURN);

}


// ----------------------------------------------------------------------------
// Schaltet in den Modus Short_Instruction_Set (Microcontroller-Befehlssatz)
// Das muss die erste Aktion nach der Initialisierung sein.
uint8_t VNC_switch_SCS(void)
{
VNC_spi_write('S');								// Sendet den String 'SCS'
VNC_spi_write('C');
VNC_spi_write('S');
VNC_spi_write(CHR_RETURN);

return(VNC_wait_for_prompt());				// auf die Antwort warten und weiterreichen
}

// ----------------------------------------------------------------------------
// sendet ein SYNCH ('E') und wartet auf die Antwort 'E' - oder auf ein Timeout
// Rckgabe: TRUE  wenn erfolgreich
//           FALSE wenn nicht erfolgreich

uint8_t VNC_synch(void)
{
VNC_spi_write(CHR_SYNCH);						// ein 'E' senden
VNC_spi_write(CHR_RETURN);						// Return als Abschluss

return(VNC_wait_for_prompt());				// hier wird auf PROMPT und SYNCH geprft 
}	

// ----------------------------------------------------------------------------
// Testet, ob eine Datei existiert
// dazu wird versucht sie zu ffnen, entsteht kein Fehler, dann ist sie vorhanden.
// Die Fehlermeldung kann per VNC_get_error() abgeholt werden.
// Rckgabe: TRUE  wenn vorhanden
//           FALSE wenn nicht vorhanden
//#define DEBUG_R
uint8_t VNC_file_exists(void)
{
register uint8_t x;

VNC_file_access(FILE_READ);					// Datei zum Lesen ffnen
x = VNC_error;

VNC_file_access(FILE_CLOSE);					// Datei schlieen, wenn sie nicht existiert
if (x) return(FALSE);							// dann gibt es einen Fehler
else return(TRUE);
}

// ----------------------------------------------------------------------------
// exportiert die lokale Variable VNC_error
uint8_t VNC_get_error(void)
{
return(VNC_error);
}

// ----------------------------------------------------------------------------
// Sendet ein bergebenes Kommando 
// wenn cmd = 0, dann wird nur ein RETURN gesendet (= EMPTY cOMMAND)
// Rckgabe: TRUE  wenn erfolgreich
//           FALSE wenn nicht erfolgreich
uint8_t VNC_send_cmd(uint8_t cmd)
{
if (cmd) VNC_spi_write(cmd);				// das CMD senden, sofern cmd > 0
VNC_spi_write(CHR_RETURN);					// dann noch ein RETURN hinterher
	
return(VNC_wait_for_prompt());			// auf die Antwort warten
}


// ----------------------------------------------------------------------------
// Legt ein Unterverzeichnis an mit dem Namen, der in der Variablen 'VNC_dirname' steht
// Rckgabe: TRUE  wenn erfolgreich
//           FALSE wenn nicht erfolgreich

uint8_t VNC_md(void)
{
uint8_t f_time;								// Hilfsvariable fr Filetime

VNC_spi_write(VNC_MKD);						// Befehl	
VNC_spi_write(CHR_SPACE);					// das Leerzeichen
VNC_spi_write_s(VNC_dirname);				// den Verzeichnisnamen 'VNC_dirname' senden

VNC_spi_write(CHR_SPACE);					// ein SPACE senden

f_time  = time[4] /  8;						// Monat >>3
f_time += time[5] *  2;						// Jahr   <1 addieren
VNC_spi_write(f_time);
	
f_time  = time[4] * 32;						// Monat <<5
f_time += time[3];							// Tage addieren
VNC_spi_write(f_time);
	
f_time  = time[1] /  8;						// Minuten >>3
f_time += time[2] *  8;						// Stunden <<3 addieren	
VNC_spi_write(f_time);
	
f_time  = time[1] * 32;						// die ersten 3Bit der Minuten <<5
f_time += time[0] /  2;						// Sekunden /2 addieren (die Auflsung betrgt 2 Sekunden (5 Bit)!)	
VNC_spi_write(f_time);

VNC_spi_write(CHR_RETURN);					// dann noch ein RETURN senden	

return(VNC_wait_for_prompt());
}

// ----------------------------------------------------------------------------
// Lscht das Unterverzeichnis mit dem Namen, der in der Variabeln 'VNC_dirname' steht
// Nicht mglich, wenn Dateien im Unterverzeichnis vorhanden sind.
// Rckgabe: TRUE  wenn erfolgreich
//           FALSE wenn nicht erfolgreich
uint8_t VNC_rd(void)
{

VNC_spi_write(VNC_DLD);						// Befehl	
VNC_spi_write(CHR_SPACE);					// das Leerzeichen
VNC_spi_write_s(VNC_dirname);				// den Verzeichnisnamen 'VNC_dirname' senden
VNC_spi_write(CHR_RETURN);					// dann noch ein RETURN senden	

return(VNC_wait_for_prompt());
}


// ----------------------------------------------------------------------------
// Sendet einen Datenstring (z.B. Verzeichnis- oder Dateinamen)
void VNC_spi_write_s(char *string)
{
while(*string) VNC_spi_write(*string++);
}

// ----------------------------------------------------------------------------
// Wechselt in ein Unterverzeichnis mit dem Namen, der in der Variabeln 'VNC_dirname' 
// steht ... oder in das bergeordnete Verzeichis
// Parameter UP   -> wechselt aufwrts in Richtung root ('..')
//           DOWN -> wechselt nach 'VNC_dirname'
// Rckgabe: TRUE  wenn erfolgreich
//           FALSE wenn nicht erfolgreich
uint8_t VNC_cd(uint8_t direction)
{

VNC_spi_write(VNC_CD);						// Instruction	
VNC_spi_write(CHR_SPACE);					// SPACE
if (direction == UP)
	{
	VNC_spi_write_s("..");					// den Verzeichnisnamen '..' senden
	}
else
	{
	VNC_spi_write_s(VNC_dirname);			// den Verzeichnisnamen aus 'VNC_dirname' senden
	}
VNC_spi_write(CHR_RETURN);					// dann noch ein RETURN senden	

return(VNC_wait_for_prompt());
}

// ----------------------------------------------------------------------------
// ffnet eine Datei zum Lesen oder Schreiben oder 
// schliet oder lscht eine Datei.
// Die Parameterwerte entsprechen dem zugehrigen binren Kommando.
// Der Name der Datei steht in der Variablen 'VNC_filename'.
// Es kann immer nur eine einzige Datei geffnet sein !
// Parameter:	- FILE_WRITE
// 				- FILE_READ
// 				- FILE_CLOSE
// 				- FILE_DELETE
// Rckgabe: TRUE  wenn erfolgreich
//           FALSE wenn nicht erfolgreich

uint8_t VNC_file_access(uint8_t modus)
{
uint8_t f_time;								// Hilfsvariable fr Filetime

VNC_spi_write(modus);						// der Parameter entspricht dem zugehrigen Kommando
VNC_spi_write(CHR_SPACE);					// das Leerzeichen
VNC_spi_write_s(VNC_filename);			// den Dateinamen senden

if (modus == FILE_WRITE)					// Das Datum nur beim ffnen zum Schreiben senden
	{
	VNC_spi_write(CHR_SPACE);				// ein zustzliches SPACE senden

	f_time  = time[4] /  8;					// Monat >>3
	f_time += time[5] *  2;					// Jahr   <1 addieren
	VNC_spi_write(f_time);
	
	f_time  = time[4] * 32;					// Monat <<5
	f_time += time[3];						// Tage addieren
	VNC_spi_write(f_time);
	
	f_time  = time[1] /  8;					// Minuten >>3
	f_time += time[2] *  8;					// Stunden <<3 addieren	
	VNC_spi_write(f_time);
	
	f_time  = time[1] * 32;					// die ersten 3Bit der Minuten <<5
	f_time += time[0] /  2;					// Sekunden /2 addieren (die Auflsung betrgt 2 Sekunden (5 Bit)!)	
	VNC_spi_write(f_time);
	}

VNC_spi_write(CHR_RETURN);					// dann noch ein RETURN senden

return(VNC_wait_for_prompt());			// auf eine Rckmeldung warten
}

// ----------------------------------------------------------------------------
// Schreibt Daten aus einem Ringpuffer in eine geffnete Datei
// Die Anzahl ist in dieser Funktion (willkrlich) auf 2^16 beschrnkt !
// Es muss genau die angegebene Anzahl von Bytes gesendet werden.
// Rckgabe: TRUE  wenn erfolgreich
//           FALSE wenn nicht erfolgreich

uint8_t VNC_write_data(uint16_t cnt)
{
uint16_t i = 0;

VNC_timeout = 30;

VNC_spi_write(VNC_WRF);						// Instruction
VNC_spi_write(CHR_SPACE);					// space
VNC_spi_write(0);								// Anzahl Bytes 1/4 (hchstwertiges Byte)
VNC_spi_write(0);								// Anzahl Bytes 2/4
VNC_spi_write((uint8_t) (cnt/256));		// Anzahl Bytes 3/4
VNC_spi_write((uint8_t) cnt);				// Anzahl Bytes 4/4 (niedrigstwertiges Byte)

VNC_spi_write(CHR_RETURN);					// dann noch ein RETURN senden

while(i < cnt) 
	{
	if (VNC_spi_write(vnc_buffer[vnc_tail]))
		{
		if (++vnc_tail > (VNC_BUFFER_SIZE - 1)) vnc_tail = 0;
		i++;
		}
	if (!(VNC_timeout)) break;
	}

return(VNC_wait_for_prompt());			// wartet auf Antwort und gibt Erfolgsmeldung zurck
}

// ---------------------------------------------------------------------------
// liest cnt-Bytes aus der bereits zum Lesen geffneten Datei 'VNC_filename'
// in einen Ringpuffer ein
// hier (willkrlich) beschrnkt auf 2^16 zu lesende Bytes
// Rckgabe: TRUE  wenn erfolgreich
//           FALSE wenn nicht erfolgreich

uint8_t VNC_read_data(uint16_t cnt)
{
uint16_t i;

VNC_timeout = 15;

VNC_spi_write(VNC_RDF);							// Instruction
VNC_spi_write(CHR_SPACE);						// Space

VNC_spi_write(0);									// Anzahl Bytes 1/4 (hchstwertiges Byte)
VNC_spi_write(0);									// Anzahl Bytes 2/4
VNC_spi_write((uint8_t) (cnt/256));			// Anzahl Bytes 3/4
VNC_spi_write((uint8_t) cnt);					// Anzahl Bytes 4/4 (niedrigstwertiges Byte)
VNC_spi_write(CHR_RETURN);						// dann noch ein RETURN senden
	
i = 0;
while (i < cnt)
	{
	if (VNC_spi_read())
		{
		vnc_buffer[vnc_head++] = spi_in;
		if (vnc_head > (VNC_BUFFER_SIZE - 1)) vnc_head = 0;
		i++;
		}
	if (!(VNC_timeout)) break;
	}
// nach den Daten folgt nur ein RETURN - oder 2 Zeichen Fehlercode + RETURN
// dazu ist in VNC_wait_for_prompt() eine spezielle Abfrage eingebaut
return(VNC_wait_for_prompt());				// gibt die Erfolgsmeldung zurck
}


// ----------------------------------------------------------------------------
// Holt die Dateigre der Datei 'VNC_filename' und schreibt sie in die globale
// Variable 'VNC_fs'.
// Auf die Union 'VNC_fs' kann via word oder byte zugegriffen werden:
// VNC_fs.word[0..1], VNC_fs.byte[0..3]
//
// Es handelt sich eigentlich um das Kommando 'DIR filename', das als Antwort zurckgibt:
// - wenn Datei vorhanden: den Dateinamen und die Dateigre
// - wenn Datei nicht vorhanden:  CR + 'C' + 'F' + CR 
//
// Eine berprfung des Erfolges kann nicht mit VNC_get_prompt() erfolgen.
// Rckgabe: TRUE  wenn erfolgreich
//           FALSE wenn nicht erfolgreich

uint8_t VNC_get_filesize(void)
{
uint8_t i = 0, j = 0;
uint8_t size[7];								// zum Einlesen der Dateigre und der folgenden 3 Zeichen
uint8_t name[15];								// zum Einlesen des Dateinamens (CR + 8 + . + 3 + CR) = 14

VNC_timeout = 10;								// Abbruch nach ca. 330ms ohne eine vollstndige Anwort

VNC_error = VNC_ERR_TO;						// Timeout error als Vorgabe
VNC_fs.dword[0] = 0;							// Dateigre auf 0 

VNC_spi_write(VNC_DIR);						// Instruction senden
VNC_spi_write(CHR_SPACE);					// Space senden
VNC_spi_write_s(VNC_filename);			// der Name der Datei steht in VNC_filename
VNC_spi_write(CHR_RETURN);					// dann noch ein RETURN senden:

// die Antwort : CR + DATEINAME + CR + BYTE1 + BYTE2+ BYTE3 + BYTE4 + CR + '>' + CR
// bei Fehler  : CR + C + F + CR

while(VNC_filename[j]) j++;				// die Lnge des Dateinamens bestimmen
j += 2;											// 2 addieren fr fhrendes und folgendes CR

while (i < j)									// CR + Dateiname + CR empfangen
	{
	if (!(VNC_timeout)) break;				// bei timeout beenden
	if (VNC_spi_read()) name[i++] = spi_in;
	}
	name[i] = 0;
													// Prfen, ob CR + C + F + CR empfangen wurde
i = (name[0] + name[1] + name[2] + name[3]);
if (i == (CHR_RETURN + 'C' + 'F' + CHR_RETURN)) // wenn ja, dann beenden
	{
	VNC_error = VNC_ERR_CF;
	return(FALSE);
	}

i = 0;											// nun kommen 4 Byte Daten mit der Dateigre
while (i < 7)									// + CR + > + CR = 7 Byte
	{												
	if (!(VNC_timeout)) break;				// bei timeout beenden
   if (VNC_spi_read())
		{
		size[i++] = spi_in;					// Inhalt von spi_in speichern
		}
	}

size[5] += size[6];							// nur die Summe von Byte5/6 prfen 
if ((VNC_timeout) && (size[5] == (CHR_PROMPT + CHR_RETURN)))
	{
	VNC_fs.byte[0] = size[0];			// low und high byte jeweils vertauscht
	VNC_fs.byte[1] = size[1];
	VNC_fs.byte[2] = size[2];
	VNC_fs.byte[3] = size[3];

	VNC_error = VNC_NO_ERR;
	}

else
	{
	VNC_error = VNC_ERR;						// Fehler
	}

if (!(VNC_error)) return(TRUE); else return(FALSE);
} 


// ----------------------------------------------------------------------------
// wird nach einem Befehl aufgerufen und wartet auf das Prompt/SYNCH 
// oder den Fehlercode (zwei cHAR + RETURN = 3 Byte)
//
// Voraussetzung ist, das der VDIP im SCS-Modus luft und als Fehlermeldung
// nur zwei Buchstaben zurckliefert.
// Andernfalls kann die Fehlermeldung nicht analysiert werden und es wird
// der Fehler VNC_ERR zurckgegeben!
// Wenn VNC_timeout == 0, dann wird beendet
// der Fehlercode wird geschrieben nach VNC_error: 	0 = kein Fehler
// Rckgabe: TRUE  wenn erfolgreich
//           FALSE wenn nicht erfolgreich

uint8_t VNC_wait_for_prompt(void)
{
uint8_t buffer[3] = {' ',' ',' '};			// mit Leerzeichen vorbelegen
VNC_timeout = 10;									// Abbruch nach ca. 330ms	ohne Anwort				
VNC_error = 0;										// Vorgabe: kein Fehler

do														// Warteschleife bis zum Empfang der Anwort
{
if (VNC_spi_read())								// Wenn ein Zeichen erfolgreich empfangen wurde
	{
	buffer[2] = buffer[1];						// dann von links nach rechts in den Puffer schieben
	buffer[1] = buffer[0];
	buffer[0] = spi_in;							// das neueste Zeichen steht in buffer[0]
	}
} while ((!(buffer[0] == CHR_RETURN)) && VNC_timeout);	// wenn buffer[0] = RETURN, 
														// kann der Inhalt des Puffers analysiert werden

if (!(VNC_timeout))								// keine Rckmeldung erhalten
	{
	VNC_error =  VNC_ERR_TO;					// Timeout Fehler
	return(FALSE);
	}
// Dass nun in Buffer[0] Return steht, das ist bekannt
// Wenn in Buffer[0] Return steht und
// wenn an zweiter Stelle 'E' oder '>' steht, dann ist das gut und zeugt vom Erfolg
if ((buffer[1] == CHR_PROMPT) || (buffer[1] == CHR_SYNCH)) return(TRUE);

// wenn an erster und zweiter Stelle ' ' steht, dann ist das die positive Quittung
// nach dem VNC_read_data() ist dann nur ein CHR_RETURN empfangen worden
else if ((buffer[1] == ' ') && (buffer[2] == ' ')) return(TRUE);
	
else													// auf Fehlercodes testen, dazu die ASCII-Werte 
	{													// von Buffer[0] ...[2] addieren
	VNC_error = buffer[0] + buffer[1] + buffer[2];

	if 		(VNC_error == ('B' + 'C' + CHR_RETURN)) VNC_error = VNC_ERR_BC;
	else if	(VNC_error == ('C' + 'F' + CHR_RETURN)) VNC_error = VNC_ERR_CF;
	else if	(VNC_error == ('N' + 'D' + CHR_RETURN)) VNC_error = VNC_ERR_ND;
	else if	(VNC_error == ('N' + 'E' + CHR_RETURN)) VNC_error = VNC_ERR_NE;
	else if	(VNC_error == ('R' + 'O' + CHR_RETURN)) VNC_error = VNC_ERR_RO;
	else if	(VNC_error == ('F' + 'I' + CHR_RETURN)) VNC_error = VNC_ERR_FI;
	else if	(VNC_error == ('D' + 'F' + CHR_RETURN)) VNC_error = VNC_ERR_DF;
	else if	(VNC_error == ('F' + 'O' + CHR_RETURN)) VNC_error = VNC_ERR_F0;
	else	    VNC_error = VNC_ERR;

	return(FALSE);
	}
}

// ----------------------------------------------------------------------------
// Startet den Suspend-Modus, die Stromaufnahme geht (mit Stick) von ca. 80mA aufca. 50mA zurck
// Rckgabe: TRUE  wenn erfolgreich
//           FALSE wenn nicht erfolgreich
uint8_t VNC_suspend(void)
{
return (VNC_send_cmd(VNC_SUD));
}

// ----------------------------------------------------------------------------
// Beendet den Suspend-Modus
// Rckgabe: TRUE  wenn erfolgreich
//           FALSE wenn nicht erfolgreich
uint8_t VNC_wake_up(void)
{
return(VNC_send_cmd(VNC_WKD));
}

// ----------------------------------------------------------------------------
// variiert den Standard-Dateinamen durch einen Zhler (0-255) 
// die Ziffern werden aus den 3 Ziffern von filenumber bernommen
// -> FILE_xxx.TXT mit xxx = 000 bis 255
// -> DIR__xxx
void VNC_make_name(uint8_t filenumber, uint8_t typ)
{
char *pointer;
uint8_t i = 0;

if (typ == FILE)
	{
	pointer = &VNC_filename[0]; 
	}
else if (typ == DIR)
	{
	pointer = &VNC_dirname[0];
	}
else return;

while (filenumber > 99)
	{
	filenumber -= 100;
	i++;
	}
pointer[5] = ('0' + i);

i = 0;
while (filenumber > 9)
	{
	filenumber -= 10;
	i++;
	}
pointer[6] = ('0' + i);

i = 0;
while (filenumber > 0)
	{
	filenumber--;
	i++;
	}
pointer[7] = ('0' + i);
}


//----------------------------------------------------------------------------
// springt an eine absolute Position innerhalb einer geffneten Datei 
// Fhrt zu Fehlermeldung bei leerer Datei !
// Rckgabe: TRUE  wenn erfolgreich
//           FALSE wenn nicht erfolgreich
uint8_t VNC_seek(uint32_t pos)
{
VNC_spi_write(VNC_SEK);						// Instruction
VNC_spi_write(CHR_SPACE);					// Space

VNC_spi_write((uint8_t) (pos>>16)>>8);	// Anzahl Bytes 1/4 (hchstwertiges Byte)
VNC_spi_write((uint8_t) (pos>>16));		// Anzahl Bytes 2/4
VNC_spi_write((uint8_t) (pos>>8));		// Anzahl Bytes 3/4
VNC_spi_write((uint8_t)  pos);			// Anzahl Bytes 4/4 (niedrigstwertiges Byte)
VNC_spi_write(CHR_RETURN);					// dann noch ein RETURN senden
	
return(VNC_wait_for_prompt());	
}


// ----------------------------------------------------------------------------
// Kopiert die Datei 'source' auf die Datei 'target'.
// Danach wird blockweise aus der Quelldatei in das Array rx_buffer gelesen
// und anschlieend in die Zieldatei geschrieben.
// Die Zieldatei (falls sie existiert) wird zu Beginn gelscht (also berschrieben).
void VNC_copy_file(uint8_t source, uint8_t target)
{
uint32_t pointer = 0;
uint32_t fs;
VNC_make_name(source, FILE);

VNC_get_filesize();							// Dateigre bestimmen
if (!(VNC_fs.dword[0])) return;			// wenn == 0 -> Datei leer
if (VNC_error) return;						// auch bei Fehler beenden (Datei fehlt z.B.)

fs = VNC_fs.dword[0];						// Filesize 32 Bit nach fs

VNC_make_name(target, FILE);				// den Namen der Zieldatei erzeugen
VNC_file_access(FILE_CLOSE);				// prophylaktisch schlieen
VNC_file_access(FILE_DELETE);				// Zieldatei lschen, sofern sie existiert
													// dann ist kein seek in der Zieldatei erforderlich
													// keine Fehlermeldung prfen
while (fs)
	{
	VNC_make_name(source, FILE);			// den Namen der Quelldatei erzeugen

	VNC_file_access(FILE_READ);			// Quelldatei zu lesen ffnen
	VNC_seek(pointer);
	
	if (fs >= VNC_BLOCK_SIZE)				// Blockweise einlesen
		{
		VNC_read_data(VNC_BLOCK_SIZE);	// Daten von source in rx_buffer kopieren
		}

	else
		{
		VNC_read_data(fs);					// letzter Rest der Datei
		}

	if(!(VNC_file_access(FILE_CLOSE))) return;	// Datei schlieen, beenden bei Fehler		
	
	VNC_make_name(target, FILE);			// den Namen der Zieldatei erzeugen
	VNC_file_access(FILE_WRITE);			// Zieldatei ffnen, Daten hinten anhngen
	
//	VNC_seek(pointer - 1);					// nicht erforderlich, da immer ans Ende angefgt wird 
	VNC_synch();								// sollte nicht erforderlich sein ???
	LED_TOG;										// Aktivittsanzeige mit der Status-LED
	
	if (fs >= VNC_BLOCK_SIZE)				// Blockweise schreiben
		{
		VNC_write_data(VNC_BLOCK_SIZE);	// rx_buffer senden
		fs -= VNC_BLOCK_SIZE;				// Filesize decrementieren
		pointer += VNC_BLOCK_SIZE;			// den Pointer incrementieren
		}
		
	else
		{
		VNC_write_data(fs);					// den Rest schreiben
		fs = 0;									// Filesize auf 0, beendet die Schleife
		}
	
	if (!(VNC_file_access(FILE_CLOSE))) return;// Bei Fehler auch vorzeitig abbrechen	
	} 
} 


// ----------------------------------------------------------------------------
// Testet, ob der Stick entnommen worden ist.
// Beim Entfernen der Disk wird gesendet: DR2 + CR + ND + CR
// Vereinfachte Lsung:
// Wenn ein EMPTY COMMAND ein NO disk zurckliefert,
// dann wird zuerst der Sendespeicher des VDIP leergesaugt,
// dann noch ein EMPTY COMMAND hinterhergeschickt.
// Wenn dann ein Fehler vorliegt, dann wurde der Stick entfernt.
// Rckgabe: liefert TRUE,  wenn kein Stick gefunden (wenn der Stick entfernt wurde)
//           liefert FALSE, wenn der Stick gefunden (wenn der Stick nicht entfert wurde)

uint8_t VNC_check_stick_removed(void)
{
VNC_send_cmd(VNC_EMPTY_CMD);			// Stick testen
if(VNC_error)								// wenn ein Fehler gemeldet wird, dann wird der Stick
	{											// entfernt worden sein.
	VNC_timeout = 5;
	while (VNC_timeout)					// 1/6 Sekunde lang Daten einlesen 
		{
		(VNC_spi_read());					// Daten einlesen und ignorieren
		}		
		
	VNC_send_cmd(VNC_EMPTY_CMD);		// Disk testen
	if (VNC_error) 						// wenn NoDisk, dann wurde die Disk abgezogen
		{
		return(TRUE);
		}
	}
return(FALSE);
} 

// ----------------------------------------------------------------------------
// Wird aufgerufen, wenn der Stick entnommen worden ist und geprft werden soll,
// ob ein neuer Stick gefunden wird.
// Beim Anstecken wird gesendet: 'DD2' + CR 'NU'+ CR + CHR_Prompt + CR.

// vereinfachte Lsung:
// es wird ein Empty_Command abgesetzt.
// Sobald keine Fehlermeldung mehr vorliegt, wird der Datenmll ausgelesen
// (die obige Meldung), der Inhalt wird aber nicht geprft.
// Statt dessen wird der Stick mit einem weiteren EMPTY COMMAND abgefragt: 
// gibt es keine Fehlermeldung, dann ist er prsent.
// Rckgabe: liefert TRUE,  wenn Stick gefunden wurde (wenn der Stick wieder angesteckt wurde)
//           liefert FALSE, wenn kein Stick gefunden wurde

uint8_t VNC_check_stick_inserted(void)
{
	VNC_send_cmd(VNC_EMPTY_CMD);			// Stick testen
	if(!(VNC_error))							// wenn kein Fehler gemeldet wird
		{
		VNC_timeout = 5;
		while (VNC_timeout)					// 1/6 Sekunde lang Daten einlesen 
			{
			(VNC_spi_read());					// Daten einlesen und ignorieren
			}

		VNC_send_cmd(VNC_EMPTY_CMD);		// Noch einmal Testen, ob ein Stick gefunden
		if (!(VNC_error)) return (TRUE);	// wenn ja, dann TRUE zurckgeben
		}
	return (FALSE);

} 

/*
// ---------------------------------------------------------------------------
// bergibt einen Filenamen an die lokale Variable 'VNC_filename'

void VNC_set_filename(char *file)
{

while (*VNC_filename++ = *file++);
uint8_t i = 0;

while (file[i])
	{
	VNC_filename[i] = file[i];
	i++;
	} 
VNC_filename[i] = 0;
} */

// ----------------------------------------------------------------------------
// TIMER0_OVF:  3.686400 MHZ / (1024 * 120) -> 33ms  Zhlt VNC_timeout herunter auf 0
// als Grundlage fr die Zeitzhlung muss ein Takt gewhlt werden, aus dem sich
// der Sekundentakt ableiten lsst: Die ISR wird 30x /Sekunde aufgerufen
ISR(TIMER0_COMPA_vect)
{
static uint8_t tick; 

if(VNC_timeout) VNC_timeout--;					// timeout runterzhlen
if(LED_timeout) LED_timeout--;					// timeout runterzhlen

tick--;
if (!(tick))											// wenn 1 Sekunde um ist (tick = 30)
	{												
	tick = 30;
	time[0]++;
	if (time[0] > 59)
		{
		time[0] = 0;
		time[1]++;
		if (time[1] > 59)
			{
			time[1] = 0;
			time[2]++;
			if (time[2] > 23)
				{
				time[3] = 0;
				time[4]++;						
				if (time[4] > 31)						// vereinfacht: 
					{										// nicht entsprechend der Anzahl der Tage eines Monats
					time[4] = 1;						// Tag = 01
					time[5]++;							// nur bis zum Monat addieren
					}
				}
			}
		}
	}
}

// --- [ eof ] ----------------------------------------------------------------